"""A module to fetch, read, and filter National Weather Service 
products issued using the Common Alerting Protocol (CAP)

.. moduleauthor:: Phil Fenstermacher <phillip.fenstermacher@gmail.com>

"""

from CAP_Feed import CAP_Feed
