from distutils.core import setup

setup(name = 'nws_cap',
      version = '0.0.1',
      description = 'NWS CAP Fetcher and Parser',
      author = 'Phil Fenstermacher',
      author_email = 'phillip.fenstermacher@gmail.com',
      url = 'https://github.com/pcfens/nws_cap/',
      packages = ['nws_cap'],
      license = 'GPL',
)
